/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */

#ifndef LIST_H_
#define LIST_H_

#include <vector>

#include <boost/shared_ptr.hpp>

#include "Type.h"

namespace airobots {

/**
 * A special deleter for boost smart pointers that does not delete the
 * contents of the data because they are already managed by another smart
 * pointer.
 */
template<class P>
struct smart_pointer_deleter {
private:
	P p_;

public:

	inline smart_pointer_deleter(P const & p) :
			p_(p) {
	}

	inline void operator()(void const *) {
		p_.reset();
	}

	inline P const & get() const {
		return p_;
	}
};

class BadSharedPtrCast: public std::exception {
public:

	BadSharedPtrCast(std::string message) :
			message(message) {
	}

	const char* what() const throw () {
		return message.c_str();
	}

	virtual ~BadSharedPtrCast() throw () {

	}

private:
	std::string message;

};

/**
 * Makes a boost::shared_ptr with a different type from another shared_ptr.
 * Therefore dynamic_cast is used.
 *
 * @throws BadSharedPtrCast if types cannot be converted
 */
template<class A, class X>
inline static boost::shared_ptr<A> make_shared_from_another(
		boost::shared_ptr<X> qx) {
	A* newContents = dynamic_cast<A*>(qx.get());if (!newContents) {
		std::stringstream msg;
		msg << "Unable to convert type " << typeid(qx.get()).name()
		<< " to type " << typeid(newContents).name() << ".";
		throw BadSharedPtrCast(msg.str());
	}
	boost::shared_ptr<A> px(newContents, smart_pointer_deleter<
			boost::shared_ptr<X> > (qx));
	return px;
}

	/**
	 * A generic list of elements. s can be all classes that derive from
	 * <code></code> and provide xmltio convert and extract methods.
	 *
	 * Internally all operations are done using boost::shared_ptr. This enables
	 * real inheritance with re-interpretation of the contents using dynamic_cast.
	 *
	 * Attention: This class cannot be extracted using the typical xmltio extract
	 * methods. Instead <code>xmltio::extract_list</code> must be used!
	 *
	 */
template<typename T>
class List: public Type, public std::vector<boost::shared_ptr<T> > {

public:

	static std::string getBaseTag() {
		return "LIST";
	}

	static std::string getBaseTag(std::string elementBaseTag) {
		return elementBaseTag + "LIST";
	}

	static std::string getTemplate() {
		return std::string("<") + getBaseTag() + " type=\"\">" + "<ELEMENT_TYPE/><GENERATOR/>"
				+ Timestamp::getTemplate() + "</" + getBaseTag() + ">";
	}

	static std::string getTemplate(std::string elementBaseTag) {
		return std::string("<") + getBaseTag(elementBaseTag) + " type=\"\">"
				+ "<ELEMENT_TYPE/><GENERATOR/>" + Timestamp::getTemplate() + "</"
				+ getBaseTag(elementBaseTag) + ">";
	}

	List() {
	}

	/**
	 * Conversion constructor that enables something like real inheritance with
	 * sub-classes of <code></code>.
	 *
	 * @throws BadSharedPtrCast if types cannot be converted
	 */
	template<typename F>
	List(List<F> list) :
			Type(list.getGenerator(), list.getTimestamp()), std::vector<
					boost::shared_ptr<T> >(list.getElementsAsType<T>()) {
	}

	virtual ~List() {

	}

	/**
	 * Returns the contents of this element list as another type that can be
	 * achieved via dynamic_cast.
	 *
	 * @throws BadSharedPtrCast if T cannot be casted to F
	 */
	template<typename F>
	std::vector<boost::shared_ptr<F> > getElementsAsType() const {

		std::vector<boost::shared_ptr<F> > vect;
		typename std::vector<boost::shared_ptr<T> >::const_iterator it =
				this->begin();
		for (; it != this->end(); ++it) {
			boost::shared_ptr<F> f = make_shared_from_another<F, T>(*it);
			vect.push_back(f);
		}
		return vect;

	}

	void sanitizeLocation(xmltio::Location &loc) const {

		// sanitize location
		xmltio::XPath xp(T::getBaseTag());
		while (xmltio::XPathIterator i = xp.evaluate(loc)) {
			xmltio::Location l(*i);
			loc.remove(l);
		}
	}

	void fillInto(xmltio::Location &loc) const {
                loc[xmltio::XPath("ELEMENT_TYPE")] = T::getJavaClassName();
		loc[xmltio::XPath("GENERATOR")] = getGenerator();
		loc[xmltio::XPath(airobots::Timestamp::getBaseTag())] = getTimestamp();

		loc["type"] = T::getBaseTag();

		typename std::vector<boost::shared_ptr<T> >::const_iterator it;
		for (it = this->begin(); it != this->end(); ++it) {
			xmltio::Location elementLoc(T::getTemplate(), T::getBaseTag());
			elementLoc = *(it->get());
			loc.add(elementLoc);
		}
	}

	static void fromLocation(const xmltio::Location &loc, List<T> &type) {

		airobots::Type::fromLocation(loc, type);

		for (xmltio::XPathIterator i = xmltio::XPath(T::getBaseTag()).evaluate(
				loc); i; ++i) {
			xmltio::Location l(*i);

			T rawT = xmltio::extract<T>(l);
			type.push_back(boost::shared_ptr<T>(new T(rawT)));

		}
	}

};

}

#endif /* LIST_H_ */
