/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */

/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; c-file-offsets: ((substatement-open . 0)) -*- */
/**
 * @file     navgoaldata.cc
 * @author   Roland Philippsen
 * @date     14.09.2006
 * @brief    Converting and extracting navgoal data to and from xml-structure
 */

#include <sstream>
#include <math.h>

#include "NavigationGoalData.h"

using namespace std;
using namespace xmltio;

namespace airobots {

std::string NavigationGoalData::getBaseTag() {
	return "NAVIGATIONGOAL";
}

std::string NavigationGoalData::getTemplate() {

	return "<" + getBaseTag() + ">" + "<GENERATOR/>"
			+ airobots::Timestamp::getTemplate()
			+ "<POSITION ref=\"world\" kind=\"absolute\" x=\"\" y=\"\" theta=\"\" />"
			+ "<TOLERANCES><DR quant=\"\"/><DTHETA quant=\"\"/></TOLERANCES>"
			+ "<TYPE/>" + "</" + getBaseTag() + ">";

}

NavigationGoalData::NavigationGoalData() {

}

NavigationGoalData::NavigationGoalData(std::string generator, float x, float y,
		float theta, float dr, float dtheta,
		NavigationGoalData::NavgoalType type, LengthUnit lU, AngleUnit aU) :
		PositionData(x, y, theta, lU, aU), type(type) {
	this->dTheta = UnitConverter::convert(dtheta, aU, DEFAULT_ANGLE_UNIT);
	this->dR = UnitConverter::convert(dR, aU, DEFAULT_ANGLE_UNIT);
}

float NavigationGoalData::getCoordinateTolerance(LengthUnit u) const {
	return UnitConverter::convert(dR, DEFAULT_LENGTH_UNIT, u);
}

float NavigationGoalData::getAngleTolerance(AngleUnit angleUnit) const {
	return UnitConverter::convert(dTheta, DEFAULT_ANGLE_UNIT, angleUnit);
}

NavigationGoalData::NavgoalType NavigationGoalData::getType() const {
	return type;
}

void NavigationGoalData::setCoordinateTolerance(float dR, LengthUnit u) {
	this->dR = UnitConverter::convert(dR, u, DEFAULT_LENGTH_UNIT);
}

void NavigationGoalData::setAngleTolerance(float dTheta, AngleUnit angleUnit) {
	this->dTheta = UnitConverter::convert(dTheta, angleUnit,
			DEFAULT_ANGLE_UNIT);
}

void NavigationGoalData::setType(NavgoalType type) {
	this->type = type;
}

bool NavigationGoalData::operator==(const NavigationGoalData &other) const {

	return (other.getGenerator() == getGenerator())
			&& (other.getTimestamp() == getTimestamp())
			&& (other.getX(METER) == getX(METER))
			&& (other.getY(METER) == getY(METER))
			&& (other.getTheta(RADIAN) == getTheta(RADIAN)) && (other.dR == dR)
			&& (other.dTheta == dTheta) && (other.type == type);

}

ostream& operator<<(ostream& o, NavigationGoalData const& ngd) {

	o << "NavigationGoalData(generator = " << ngd.getGenerator()
			<< ", timestamp = " << ngd.getTimestamp() << ", x = "
			<< ngd.getX(METER) << ", y = " << ngd.getY(METER) << ", theta = "
			<< ngd.getTheta(RADIAN) << ", dR = "
			<< ngd.getCoordinateTolerance(METER) << ", dTheta = "
			<< ngd.getAngleTolerance(RADIAN) << ", type = " << ngd.getType()
			<< ")";
	return o;
}

void NavigationGoalData::sanitizeLocation(xmltio::Location &loc) const {

}

void NavigationGoalData::fillInto(xmltio::Location &loc) const {

	PositionData::fillInto(loc);

	loc[XPath("TOLERANCES/DR/@quant")] = getCoordinateTolerance(
			DEFAULT_LENGTH_UNIT);
	loc[XPath("TOLERANCES/DTHETA/@quant")] = min(
			getAngleTolerance(DEFAULT_ANGLE_UNIT), (float) M_PI);

	std::string type;
	switch (getType()) {
	case NavigationGoalData::GLOBAL:
		type = "GLOBAL";
		break;
	case NavigationGoalData::LOCAL:
		type = "LOCAL";
		break;
	default:
		type = "GLOBAL";
		break;
	}
	loc[XPath("TYPE")] = type;

}

void NavigationGoalData::fromLocation(const xmltio::Location &loc_,
		NavigationGoalData &ngd) {

	PositionData::fromLocation(loc_, ngd);

	ngd.setCoordinateTolerance(
			extract<float>(loc_[XPath("TOLERANCES/DR/@quant")]),
			DEFAULT_LENGTH_UNIT);
	ngd.setAngleTolerance(
			min(extract<float>(loc_[XPath("TOLERANCES/DTHETA/@quant")]).get(),
					(float) M_PI), DEFAULT_ANGLE_UNIT);

	LocationPtr typeLoc = find(loc_, XPath("TYPE"));
	NavigationGoalData::NavgoalType type = NavigationGoalData::GLOBAL;
	if (typeLoc) {
		if (typeLoc->getTextValue() == "LOCAL") {
			type = NavigationGoalData::LOCAL;
		}
	}
	ngd.setType(type);

}
}
