/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * ROSMoveBaseConfiguration.h
 *
 *  Created on: May 31, 2012
 *      Author: pdressel
 */

#ifndef ROSMOVEBASECONFIGURATION_H_
#define ROSMOVEBASECONFIGURATION_H_

#include <stdio.h>
#include <string>
#include <xmltio/xmltio.hpp>
#include "ROSDynamicConfiguration.h"

namespace airobots 
{

class ROSMoveBaseConfiguration : public ROSDynamicConfiguration 
{
    
public:
    
    /**
     * ROSMoveBaseConfiguration constructor. Make sure the member variables are 
     * set with the current configuration as is on the corresponding ROS server.
     * @param isGlobalCostmap 
     * Setting this parameter to true allows for manipulation of global_costmap
     * parameters. Setting this parameter to false allows for manipulation of 
     * local_costmap parameters. The default value is 'true'.
     */
    ROSMoveBaseConfiguration();
    virtual ~ROSMoveBaseConfiguration() {};
    
    /**
     * Explanation: The name of the plugin for the global planner to use with
     * move_base, see pluginlib documentation for more details on plugins. This
     * plugin must adhere to the nav_core::BaseGlobalPlanner interface specified
     * in the nav_core package.<br>
     * name: '<node_name>/base_global_planner'.<br>
     * Type and default: std::string, 'navfn/NavfnROS'
     * @return the base_global_planner
     */
    std::string getBaseGlobalPlanner() {
        return this->base_global_planner;
    }

    /**
     * Explanation: The name of the plugin for the global planner to use with
     * move_base, see pluginlib documentation for more details on plugins. This
     * plugin must adhere to the nav_core::BaseGlobalPlanner interface specified
     * in the nav_core package.<br>
     * name: '<node_name>/base_global_planner'.<br>
     * Type and default: std::string, 'navfn/NavfnROS'
     * @param base_global_planner the base_global_planner to set
     */
    void setBaseGlobalPlanner(std::string base_global_planner) {
        this->base_global_planner = base_global_planner;
    }

    /**
     * Explanation: The name of the plugin for the local planner to use with move_base see
     * pluginlib documentation for more details on plugins. This plugin must
     * adhere to the nav_core::BaseLocalPlanner interface specified in the
     * nav_core package.<br>
     * name: '<node_name>/base_local_planner'.<br>
     * Type and default: std::string, 'base_local_planner/TrajectoryPlannerROS'
     * @return the base_local_planner
     */
    std::string getBaseLocalPlanner() {
        return this->base_local_planner;
    }

    /**
     * Explanation: The name of the plugin for the local planner to use with move_base see
     * pluginlib documentation for more details on plugins. This plugin must
     * adhere to the nav_core::BaseLocalPlanner interface specified in the
     * nav_core package.<br>
     * name: '<node_name>/base_local_planner'.<br>
     * Type and default: std::string, 'base_local_planner/TrajectoryPlannerROS'
     * @param base_local_planner the base_local_planner to set
     */
    void setBaseLocalPlanner(std::string base_local_planner) {
        this->base_local_planner = base_local_planner;
    }

    /**
     * Explanation: The rate in Hz at which to run the global planning loop. If the frequency
     * is set to 0.0, the global planner will only run when a new goal is
     * received or the local planner reports that its path is blocked.<br>
     * name: '<node_name>/planner_frequency'.<br>
     * Type and default: double, 0.0.
     * @return the planner_frequency
     */
    double getPlannerFrequency() {
        return this->planner_frequency;
    }

    /**
     * Explanation: The rate in Hz at which to run the global planning loop. If the frequency
     * is set to 0.0, the global planner will only run when a new goal is
     * received or the local planner reports that its path is blocked.<br>
     * name: '<node_name>/planner_frequency'.<br>
     * Type and default: double, 0.0.
     * @param planner_frequency the planner_frequency to set
     */
    void setPlannerFrequency(double planner_frequency) {
        this->planner_frequency = planner_frequency;
    }

    /**
     * Explanation: The rate in Hz at which to run the control loop and send velocity
     * commands to the base.<br>
     * name: '<node_name>/controller_frequency'.<br>
     * Type and default: double, 20.0
     * @return the controller_frequency
     */
    double getControllerFrequency() {
        return this->controller_frequency;
    }

    /**
     * Explanation: The rate in Hz at which to run the control loop and send velocity
     * commands to the base.<br>
     * name: '<node_name>/controller_frequency'.<br>
     * Type and default: double, 20.0
     * @param controller_frequency the controller_frequency to set
     */
    void setControllerFrequency(double controller_frequency) {
        this->controller_frequency = controller_frequency;
    }

    /**
     * Explanation: How long the planner will wait in seconds in an attempt to find a valid
     * plan before space-clearing operations are performed.<br>
     * name: '<node_name>/planner_patience'.<br>
     * Type and default: double, 5.0
     * @return the planner_patience
     */
    double getPlannerPatience() {
        return this->planner_patience;
    }

    /**
     * Explanation: How long the planner will wait in seconds in an attempt to find a valid
     * plan before space-clearing operations are performed.<br>
     * name: '<node_name>/planner_patience'.<br>
     * Type and default: double, 5.0
     * @param planner_patience the planner_patience to set
     */
    void setPlannerPatience(double planner_patience) {
        this->planner_patience = planner_patience;
    }

    /**
     * Explanation: How long the controller will wait in seconds without receiving a valid
     * control before space-clearing operations are performed.<br>
     * name: '<node_name>/controller_patience'.<br>
     * Type and default: double, 15.0
     * @return the controller_patience
     */
    double getControllerPatience() {
        return this->controller_patience;
    }

    /**
     * Explanation: How long the controller will wait in seconds without receiving a valid
     * control before space-clearing operations are performed.<br>
     * name: '<node_name>/controller_patience'.<br>
     * Type and default: double, 15.0
     * @param controller_patience the controller_patience to set
     */
    void setControllerPatience(double controller_patience) {
        this->controller_patience = controller_patience;
    }

    /**
     * Explanation: The distance away from the robot in meters at which obstacles will be
     * cleared from the costmap when attempting to clear space in the map. Note,
     * this parameter is only used when the default recovery behaviors are used
     * for move_base.<br>
     * name: '<node_name>/conservative_reset_dist'.<br>
     * Type and default: double, 3.0
     * @return the conservative_reset_dist
     */
    double getConservativeResetDist() {
        return this->conservative_reset_dist;
    }

    /**
     * Explanation: The distance away from the robot in meters at which obstacles will be
     * cleared from the costmap when attempting to clear space in the map. Note,
     * this parameter is only used when the default recovery behaviors are used
     * for move_base.<br>
     * name: '<node_name>/conservative_reset_dist'.<br>
     * Type and default: double, 3.0
     * @param conservative_reset_dist the conservative_reset_dist to set
     */
    void setConservativeResetDist(double conservative_reset_dist) {
        this->conservative_reset_dist = conservative_reset_dist;
    }

    /**
     * Explanation: Whether or not to enable the move_base recovery behaviors to attempt to
     * clear out space.<br>
     * name: '<node_name>/recovery_behavior_enabled'.<br>
     * Type and default: bool, true
     * @return the recovery_behavior_enabled
     */
    bool isRecoveryBehaviorEnabled() {
        return this->recovery_behavior_enabled;
    }

    /**
     * Explanation: Whether or not to enable the move_base recovery behaviors to attempt to
     * clear out space.<br>
     * name: '<node_name>/recovery_behavior_enabled'.<br>
     * Type and default: bool, true
     * @param recovery_behavior_enabled the recovery_behavior_enabled to set
     */
    void setRecoveryBehaviorEnabled(bool recovery_behavior_enabled) {
        this->recovery_behavior_enabled = recovery_behavior_enabled;
    }

    /**
     * Explanation: Determines whether or not the robot will attempt an in-place rotation
     * when attempting to clear out space. Note: This parameter is only used
     * when the default recovery behaviors are in use, meaning the user has not
     * set the recovery_behaviors parameter to anything custom. <br>
     * name: '<node_name>/clearing_rotation_allowed'.<br>
     * Type and default: bool, true
     * @return the clearing_rotation_allowed
     */
    bool isClearingRotationAllowed() {
        return this->clearing_rotation_allowed;
    }

    /**
     * Explanation: Determines whether or not the robot will attempt an in-place rotation
     * when attempting to clear out space. Note: This parameter is only used
     * when the default recovery behaviors are in use, meaning the user has not
     * set the recovery_behaviors parameter to anything custom. <br>
     * name: '<node_name>/clearing_rotation_allowed'.<br>
     * Type and default: bool, true
     * @param clearing_rotation_allowed the clearing_rotation_allowed to set
     */
    void setClearingRotationAllowed(bool clearing_rotation_allowed) {
        this->clearing_rotation_allowed = clearing_rotation_allowed;
    }

    /**
     * Explanation: Determines whether or not to shutdown the costmaps of the node when
     * move_base is in an inactive state.<br>
     * name: '<node_name>/shutdown_costmaps'.<br>
     * Type and default: bool, false
     * @return the shutdown_costmaps
     */
    bool isShutdownCostmaps() {
        return this->shutdown_costmaps;
    }

    /**
     * Explanation: Determines whether or not to shutdown the costmaps of the node when
     * move_base is in an inactive state.<br>
     * name: '<node_name>/shutdown_costmaps'.<br>
     * Type and default: bool, false
     * @param shutdown_costmaps the shutdown_costmaps to set
     */
    void setShutdownCostmaps(bool shutdown_costmaps) {
        this->shutdown_costmaps = shutdown_costmaps;
    }

    /**
     * Explanation: How long in seconds to allow for oscillation before executing recovery
     * behaviors. A value of 0.0 corresponds to an infinite timeout.<br>
     * name: '<node_name>/oscillation_timeoutr'.<br>
     * Type and default: double, 0.0
     * @return the oscillation_timeout
     */
    double getOscillationTimeout() {
        return this->oscillation_timeout;
    }

    /**
     * Explanation: How long in seconds to allow for oscillation before executing recovery
     * behaviors. A value of 0.0 corresponds to an infinite timeout.<br>
     * name: '<node_name>/oscillation_timeoutr'.<br>
     * Type and default: double, 0.0
     * @param oscillation_timeout the oscillation_timeout to set
     */
    void setOscillationTimeout(double oscillation_timeout) {
        this->oscillation_timeout = oscillation_timeout;
    }

    /**
     * Explanation: How far in meters the robot must move to be considered not to be
     * oscillating. Moving this far resets the timer counting up to the
     * ~oscillation_timeout.<br>
     * name: '<node_name>/oscillation_distance'.<br>
     * Type and default: double, 0.5
     * @return the oscillation_distance
     */
    double getOscillationDistance() {
        return this->oscillation_distance;
    }

    /**
     * Explanation: How far in meters the robot must move to be considered not to be
     * oscillating. Moving this far resets the timer counting up to the
     * ~oscillation_timeout.<br>
     * name: '<node_name>/oscillation_distance'.<br>
     * Type and default: double, 0.5
     * @param oscillation_distance the oscillation_distance to set
     */
    void setOscillationDistance(double oscillation_distance) {
        this->oscillation_distance = oscillation_distance;
    }

    /**
     * Explanation: Reset all values at the target dynamic_reconfiguration
     * server to its defaults. ROS topic name: '<node_name>/restore_defaults'.
     * Type and default: bool, false
     * @return the restore_defaults
     */
    bool isRestoreDefaults() {
        return this->restore_defaults;
    }

    /**
     * Explanation: Reset all values at the target dynamic_reconfiguration
     * server to its defaults. ROS topic name: '<node_name>/restore_defaults'.
     * Type and default: bool, false
     * @param restore_defaults the restore_defaults to set
     */
    void setRestoreDefaults(bool restore_defaults) {
        this->restore_defaults = restore_defaults;
    }
    
    static std::string getBaseTag();
    static std::string getTemplate();
    
    void sanitizeLocation(xmltio::Location &loc) const;
    void fillInto(xmltio::Location &loc) const;
    static void fromLocation(const xmltio::Location &loc, ROSMoveBaseConfiguration &config);    
    
    static const std::string BASE_GLOBAL_PLANNER_ATTRIBUTE;
    static const std::string BASE_LOCAL_PLANNER_ATTRIBUTE;
    static const std::string PLANNER_FREQUENCY_ATTRIBUTE;
    static const std::string CONTROLLER_FREQUENCY_ATTRIBUTE;
    static const std::string PLANNER_PATIENCE_ATTRIBUTE;
    static const std::string CONTROLLER_PATIENCE_ATTRIBUTE;
    static const std::string CONSERVATIVE_RESET_DIST_ATTRIBUTE;
    static const std::string RECOVERY_BEHAVIOR_ENABLED_ATTRIBUTE;
    static const std::string CLEARING_ROTATION_ALLOWED_ATTRIBUTE;
    static const std::string SHUTDOWN_COSTMAPS_ATTRIBUTE;
    static const std::string OSCILLATION_TIMEOUT_ATTRIBUTE;
    static const std::string OSCILLATION_DISTANCE_ATTRIBUTE;
    static const std::string RESTORE_DEFAULTS_ATTRIBUTE;  
    
private:
    
    static const std::string MOVEBASE_CONFIGURATION_TAG;
    
    /**
     * Explanation: The name of the plugin for the global planner to use with 
     * move_base, see pluginlib documentation for more details on plugins. This 
     * plugin must adhere to the nav_core::BaseGlobalPlanner interface specified 
     * in the nav_core package.<br>
     * name: '<node_name>/base_global_planner'.<br>
     * Type and default: string, 'navfn/NavfnROS'
     */
    std::string base_global_planner;

    /**
     * Explanation: The name of the plugin for the local planner to use with move_base see 
     * pluginlib documentation for more details on plugins. This plugin must 
     * adhere to the nav_core::BaseLocalPlanner interface specified in the 
     * nav_core package.<br>
     * name: '<node_name>/base_local_planner'.<br>
     * Type and default: string, 'base_local_planner/TrajectoryPlannerROS'
     */
    std::string base_local_planner;
    
    /**
     * Explanation: The rate in Hz at which to run the global planning loop. If the frequency
     * is set to 0.0, the global planner will only run when a new goal is 
     * received or the local planner reports that its path is blocked.<br>
     * name: '<node_name>/planner_frequency'.<br>
     * Type and default: double, 0.0.
     */
    double planner_frequency;

    /**
     * Explanation: The rate in Hz at which to run the control loop and send velocity 
     * commands to the base.<br>
     * name: '<node_name>/controller_frequency'.<br>
     * Type and default: double, 20.0
     */    
    double controller_frequency;
    
    /**
     * Explanation: How long the planner will wait in seconds in an attempt to find a valid 
     * plan before space-clearing operations are performed.<br>
     * name: '<node_name>/planner_patience'.<br>
     * Type and default: double, 5.0
     */    
    double planner_patience;

    /**
     * Explanation: How long the controller will wait in seconds without receiving a valid 
     * control before space-clearing operations are performed.<br>
     * name: '<node_name>/controller_patience'.<br>
     * Type and default: double, 15.0
     */     
    double controller_patience;

    /**
     * Explanation: The distance away from the robot in meters at which obstacles will be 
     * cleared from the costmap when attempting to clear space in the map. Note,
     * this parameter is only used when the default recovery behaviors are used 
     * for move_base.<br>
     * name: '<node_name>/conservative_reset_dist'.<br>
     * Type and default: double, 3.0
     */    
    double conservative_reset_dist;
    
    /**
     * Explanation: Whether or not to enable the move_base recovery behaviors to attempt to 
     * clear out space.<br>
     * name: '<node_name>/recovery_behavior_enabled'.<br>
     * Type and default: bool, true
     */    
    bool recovery_behavior_enabled;

    /**
     * Explanation: Determines whether or not the robot will attempt an in-place rotation 
     * when attempting to clear out space. Note: This parameter is only used 
     * when the default recovery behaviors are in use, meaning the user has not 
     * set the recovery_behaviors parameter to anything custom. <br>
     * name: '<node_name>/clearing_rotation_allowed'.<br>
     * Type and default: bool, true
     */  
    bool clearing_rotation_allowed;
    
    /**
     * Explanation: Determines whether or not to shutdown the costmaps of the node when 
     * move_base is in an inactive state.<br>
     * name: '<node_name>/shutdown_costmaps'.<br>
     * Type and default: bool, false
     */    
    bool shutdown_costmaps;
    
    /**
     * Explanation: How long in seconds to allow for oscillation before executing recovery 
     * behaviors. A value of 0.0 corresponds to an infinite timeout.<br>
     * name: '<node_name>/oscillation_timeoutr'.<br>
     * Type and default: double, 0.0
     */    
    double oscillation_timeout;
    
    /**
     * Explanation: How far in meters the robot must move to be considered not to be 
     * oscillating. Moving this far resets the timer counting up to the 
     * ~oscillation_timeout.<br>
     * name: '<node_name>/oscillation_distance'.<br>
     * Type and default: double, 0.5
     */    
    double oscillation_distance;
    
    /**
     * Explanation: Reset all values at the target dynamic_reconfiguration
     * server to its defaults. ROS topic name: '<node_name>/restore_defaults'.
     * Type and default: bool, false
     */ 
    bool restore_defaults;    
};

}

#endif /* ROSMOVEBASECONFIGURATION_H_ */