/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */

#include "UnitConverter.h"
#include <math.h>

using namespace airobots;

int UnitConverter::convert(int value, LengthUnit from, LengthUnit to) {
	float asMillimeter = convertFrom((float) value, from);
	return round(convertTo(asMillimeter, to));
}

long UnitConverter::convert(long value, LengthUnit from, LengthUnit to) {
	double asMillimeter = convertFrom((double) value, from);
	return round(convertTo(asMillimeter, to));
}

double UnitConverter::convert(double value, LengthUnit from, LengthUnit to) {
	double asMillimeter = convertFrom(value, from);
	return convertTo(asMillimeter, to);
}

float UnitConverter::convert(float value, LengthUnit from, LengthUnit to) {
	float asMillimeter = convertFrom(value, from);
	return convertTo(asMillimeter, to);
}

double UnitConverter::convert(double value, AngleUnit from, AngleUnit to) {
	double asRadiant = convertFrom(value, from);
	return convertTo(asRadiant, to);
}

float UnitConverter::convert(float value, AngleUnit from, AngleUnit to) {
	float asRadiant = convertFrom(value, from);
	return convertTo(asRadiant, to);
}

long UnitConverter::convert(long value, TimeUnit from, TimeUnit to) {
	long asMicroSeconds = convertFrom(value, from);
	return convertTo(asMicroSeconds, to);
}

double airobots::UnitConverter::convert(double value, SpeedUnit from,
		SpeedUnit to) {
	double asMPS = convertFrom(value, from);
	return convertTo(asMPS, to);
}

float airobots::UnitConverter::convert(float value, SpeedUnit from,
		SpeedUnit to) {
	float asMPS = convertFrom(value, from);
	return convertTo(asMPS, to);
}

double airobots::UnitConverter::convertFrom(double value, SpeedUnit unit) {
	switch (unit) {
	case METER_PER_SEC:
		return value;
	case KILOMETER_PER_HOUR:
		return value / 3.6;
	default:
		throw UnknownUnitException(
				"Don't know how to convert from " + unitName(unit));
	}
}

double airobots::UnitConverter::convertTo(double value, SpeedUnit unit) {
	switch (unit) {
	case METER_PER_SEC:
		return value;
	case KILOMETER_PER_HOUR:
		return value * 3.6;
	default:
		throw UnknownUnitException(
				"Don't know how to convert from " + unitName(unit));
	}
}

float airobots::UnitConverter::convertFrom(float value, SpeedUnit unit) {
	switch (unit) {
	case METER_PER_SEC:
		return value;
	case KILOMETER_PER_HOUR:
		return value / 3.6;
	default:
		throw UnknownUnitException(
				"Don't know how to convert from " + unitName(unit));
	}
}

float airobots::UnitConverter::convertTo(float value, SpeedUnit unit) {
	switch (unit) {
	case METER_PER_SEC:
		return value;
	case KILOMETER_PER_HOUR:
		return value * 3.6;
	default:
		throw UnknownUnitException(
				"Don't know how to convert from " + unitName(unit));
	}
}


double airobots::UnitConverter::convert(double value, RotationalSpeedUnit from,
		RotationalSpeedUnit to) {
	double asMPS = convertFrom(value, from);
	return convertTo(asMPS, to);
}

float airobots::UnitConverter::convert(float value, RotationalSpeedUnit from,
		RotationalSpeedUnit to) {
	float asMPS = convertFrom(value, from);
	return convertTo(asMPS, to);
}

double airobots::UnitConverter::convertFrom(double value, RotationalSpeedUnit unit) {
	switch (unit) {
	case RADIANS_PER_SEC:
		return value;
	case DEGREES_PER_SEC:
		return value / 180.0 * M_PI;
	default:
		throw UnknownUnitException(
				"Don't know how to convert from " + unitName(unit));
	}
}

double airobots::UnitConverter::convertTo(double value, RotationalSpeedUnit unit) {
	switch (unit) {
	case RADIANS_PER_SEC:
		return value;
	case DEGREES_PER_SEC:
		return value / (float) M_PI * 180.0f;
	default:
		throw UnknownUnitException(
				"Don't know how to convert from " + unitName(unit));
	}
}

float airobots::UnitConverter::convertFrom(float value, RotationalSpeedUnit unit) {
	switch (unit) {
	case RADIANS_PER_SEC:
		return value;
	case DEGREES_PER_SEC:
		return value / 180.0 * M_PI;
	default:
		throw UnknownUnitException(
				"Don't know how to convert from " + unitName(unit));
	}
}

float airobots::UnitConverter::convertTo(float value, RotationalSpeedUnit unit) {
	switch (unit) {
	case RADIANS_PER_SEC:
		return value;
	case DEGREES_PER_SEC:
		return value / (float) M_PI * 180.0f;
	default:
		throw UnknownUnitException(
				"Don't know how to convert from " + unitName(unit));
	}
}


double UnitConverter::convertFrom(double value, LengthUnit unit) {
	switch (unit) {
	case METER:
		return value * 1000.0;
	case CENTIMETER:
		return value * 10.0;
	case MILLIMETER:
		return value;
	default:
		throw UnknownUnitException(
				"Don't know how to convert from " + unitName(unit));
	}
}

double UnitConverter::convertTo(double value, LengthUnit unit) {
	switch (unit) {
	case METER:
		return value / 1000.0;
	case CENTIMETER:
		return value / 10.0;
	case MILLIMETER:
		return value;
	default:
		throw UnknownUnitException(
				"Don't know how to convert to " + unitName(unit));
	}
}

float UnitConverter::convertFrom(float value, LengthUnit unit) {
	switch (unit) {
	case METER:
		return value * 1000.0f;
	case CENTIMETER:
		return value * 10.0f;
	case MILLIMETER:
		return value;
	default:
		throw UnknownUnitException(
				"Don't know how to convert from " + unitName(unit));
	}
}

float UnitConverter::convertTo(float value, LengthUnit unit) {
	switch (unit) {
	case METER:
		return value / 1000.0f;
	case CENTIMETER:
		return value / 10.0f;
	case MILLIMETER:
		return value;
	default:
		throw UnknownUnitException(
				"Don't know how to convert to " + unitName(unit));
	}
}

double UnitConverter::convertFrom(double value, AngleUnit unit) {
	switch (unit) {
	case DEGREE:
		return value / 180.0 * M_PI;
	case RADIAN:
		return value;
	default:
		throw UnknownUnitException(
				"Don't know how to convert from " + unitName(unit));
	}
}

double UnitConverter::convertTo(double value, AngleUnit unit) {
	switch (unit) {
	case DEGREE:
		return value / M_PI * 180.0;
	case RADIAN:
		return value;
	default:
		throw UnknownUnitException(
				"Don't know how to convert to " + unitName(unit));
	}
}

float UnitConverter::convertFrom(float value, AngleUnit unit) {
	switch (unit) {
	case DEGREE:
		return value / 180.0f * (float) M_PI;
	case RADIAN:
		return value;
	default:
		throw UnknownUnitException(
				"Don't know how to convert from " + unitName(unit));
	}
}

float UnitConverter::convertTo(float value, AngleUnit unit) {
	switch (unit) {
	case DEGREE:
		return value / (float) M_PI * 180.0f;
	case RADIAN:
		return value;
	default:
		throw UnknownUnitException(
				"Don't know how to convert to " + unitName(unit));
	}
}

long UnitConverter::convertTo(long value, TimeUnit unit) {
	switch (unit) {
	case NANOSECONDS:
		return value * 1000L;
	case MICROSECONDS:
		return value;
	case MILLISECONDS:
		return value / 1000L;
	case SECONDS:
		return convertTo(value, MILLISECONDS) / 1000L;
	case MINUTES:
		return convertTo(value, SECONDS) / 60L;
	case HOURS:
		return convertTo(value, MINUTES) / 60L;
	case DAYS:
		return convertTo(value, HOURS) / 24L;
	default:
		throw UnknownUnitException(
				"Don't know how to convert to " + unitName(unit));
	}
}

long UnitConverter::convertFrom(long value, TimeUnit unit) {
	switch (unit) {
	case NANOSECONDS:
		return value / 1000L;
	case MICROSECONDS:
		return value;
	case MILLISECONDS:
		return value * 1000L;
	case SECONDS:
		return convertFrom(value, MILLISECONDS) * 1000L;
	case MINUTES:
		return convertFrom(value, SECONDS) * 60L;
	case HOURS:
		return convertFrom(value, MINUTES) * 60L;
	case DAYS:
		return convertFrom(value, HOURS) * 24L;
	default:
		throw UnknownUnitException(
				"Don't know how to convert to " + unitName(unit));
	}
}

std::string UnitConverter::unitName(LengthUnit unit) {
	switch (unit) {
	case METER:
		return "METER";
	case CENTIMETER:
		return "CENTIMETER";
	case MILLIMETER:
		return "MILLIMETER";
	default:
		return "unknown";
	}
}

std::string UnitConverter::unitName(AngleUnit unit) {
	switch (unit) {
	case DEGREE:
		return "DEGREE";
	case RADIAN:
		return "RADIANT";
	default:
		return "unknown";
	}
}

std::string UnitConverter::unitName(TimeUnit unit) {
	switch (unit) {
	case NANOSECONDS:
		return "NANOSECONDS";
	case MICROSECONDS:
		return "MICROSECONDS";
	case MILLISECONDS:
		return "MILLISECONDS";
	case SECONDS:
		return "SECONDS";
	case MINUTES:
		return "MINUTES";
	case HOURS:
		return "HOURS";
	case DAYS:
		return "DAYS";
	default:
		return "unknown";
	}
}

std::string UnitConverter::unitName(SpeedUnit speedUnit) {
	switch (speedUnit) {
	case METER_PER_SEC:
		return "METER_PER_SEC";
	case KILOMETER_PER_HOUR:
		return "KILOMETER_PER_HOUR";
	default:
		return "unknown";
	}
}

std::string UnitConverter::unitName(RotationalSpeedUnit speedUnit) {
	switch (speedUnit) {
	case RADIANS_PER_SEC:
		return "RADIANS_PER_SEC";
	case DEGREES_PER_SEC:
		return "DEGREES_PER_SEC";
	default:
		return "unknown";
	}
}
