/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.common;

import java.util.concurrent.TimeUnit;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Nodes;
import nu.xom.ParsingException;

/**
 * A timestamp with microseconds precision.
 * 
 * @author jwienke
 * @author dklotz
 */
public class MicroTimestamp {

	private long seconds = 0;
	private long microSeconds = 0;

	/**
	 * Creates a timestamp with a value of 0 seconds and 0 microseconds.
	 */
	public MicroTimestamp() {
	}

	/**
	 * Creates a timestamp with the given value, converting it from the given
	 * time unit to seconds and microseconds.
	 * 
	 * <b>WARNING:</b> This timestamp only has microseconds precision
	 * internally, so giving it a value that is smaller / more precise than 1
	 * microseconds (for example in nanoseconds) WILL loose precision.
	 * 
	 * @param value
	 *            The time value.
	 * @param unit
	 *            The time unit.
	 * @see TimeUnit
	 */
	public MicroTimestamp(long value, TimeUnit unit) {
		this.setInUnit(value, unit);
	}

	/**
	 * Creates a timestamp with the given seconds as a value.
	 * 
	 * @param seconds
	 *            The value in seconds.
	 */
	public MicroTimestamp(long seconds) {
		setSeconds(seconds);
	}

	/**
	 * Creates a timestamp with microsecond precision with the given time.
	 * 
	 * @param seconds
	 *            The timestamps seconds part.
	 * @param microSeconds
	 *            The timestamps microseconds part (i.e. only the part below
	 *            1s).
	 */
	public MicroTimestamp(long seconds, long microSeconds) {
		setSeconds(seconds);
		setMicroSeconds(microSeconds);
	}

	public long getSeconds() {
		return seconds;
	}

	public void setSeconds(long seconds) {
		this.seconds = seconds;
	}

	public long getMicroSeconds() {
		return microSeconds;
	}

	public void setMicroSeconds(long microSeconds) {
		if (microSeconds >= 1000000 || microSeconds < 0) {
			throw new IllegalArgumentException(
					"Microseconds can only be >= 0 and < 1000000");
		}
		this.microSeconds = microSeconds;
	}

	/**
	 * Gets this timestamps value in an arbitrary {@link TimeUnit}.
	 * 
	 * @param unit
	 *            The unit of time to convert this timestamp into.
	 * @return The value of this timestamp in the given unit.
	 */
	public long getInUnit(TimeUnit unit) {
		long secondsConverted = unit.convert(seconds, TimeUnit.SECONDS);
		long microSecondsConverted = unit.convert(microSeconds,
				TimeUnit.MICROSECONDS);

		return secondsConverted + microSecondsConverted;
	}

	/**
	 * Sets the timestamp to the given value, converting it from the given time
	 * unit to seconds and microseconds.
	 * 
	 * <b>WARNING:</b> This timestamp only has microseconds precision
	 * internally, so giving it a value that is smaller / more precise than 1
	 * microseconds (for example in nanoseconds) WILL loose precision.
	 * 
	 * @param value
	 *            The time value.
	 * @param unit
	 *            The time unit.
	 * @see TimeUnit
	 */
	public void setInUnit(long value, TimeUnit unit) {
		// Convert value to seconds (loosing everything below second precision)
		long secondsConverted = TimeUnit.SECONDS.convert(value, unit);
		setSeconds(secondsConverted);

		// Substract the second part from the whole value [original unit]
		long valueMicroSecondsPart = value
				- unit.convert(secondsConverted, TimeUnit.SECONDS);
		// Convert it to microseconds
		long microSecondsConverted = TimeUnit.MICROSECONDS.convert(
				valueMicroSecondsPart, unit);
		setMicroSeconds(microSecondsConverted);
	}

	public static String getBaseTag() {
		return "READTIME";
	}

	public static void fromElement(Element element, MicroTimestamp type)
			throws ParsingException {

		Nodes secondsNodes = element.query("SECONDS");
		if (secondsNodes.size() < 1
				|| !(secondsNodes.get(0) instanceof Element)) {
			throw new ParsingException("Cannot find Element with name SECONDS.");
		}
		Element secondsElement = (Element) secondsNodes.get(0);
		type.setSeconds(ElementParser.getLongAttributeValue(secondsElement,
				"value"));

		Nodes microSecondsNodes = element.query("MICROSECONDS");
		if (microSecondsNodes.size() < 1
				|| !(microSecondsNodes.get(0) instanceof Element)) {
			throw new ParsingException(
					"Cannot find Element with name MICROSECONDS.");
		}
		Element microSecondsElement = (Element) microSecondsNodes.get(0);
		type.setMicroSeconds(ElementParser.getLongAttributeValue(
				microSecondsElement, "value"));

	}

	public void fillInto(Element parent) {

		Element secondsElement = new Element("SECONDS");
		parent.appendChild(secondsElement);
		secondsElement.addAttribute(new Attribute("value", String
				.valueOf(getSeconds())));

		Element microSecondsElement = new Element("MICROSECONDS");
		parent.appendChild(microSecondsElement);
		microSecondsElement.addAttribute(new Attribute("value", String
				.valueOf(getMicroSeconds())));

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean equals(Object obj) {

		if (!(obj instanceof MicroTimestamp)) {
			return false;
		}

		MicroTimestamp other = (MicroTimestamp) obj;

		return (other.seconds == seconds)
				&& (other.microSeconds == microSeconds);

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int hashCode() {
		return (int) (42 * seconds + 75 * microSeconds);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "MicroTimestamp [seconds = " + seconds + ", microSeconds = "
				+ microSeconds + "]";
	}

}
