/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.geometry;

import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.Type;

public class BoundingBox3D extends Type {

	public static final String ORIGIN_TAG_NAME = "ORIGIN";
	public static final String ROTATION_TAG_NAME = "ROTATION";
	public static final String SIZE_TAG_NAME = "SIZE";

	private Point3D origin = new Point3D();
	private Point3D size = new Point3D();
	private Rotation3D rotation = new Rotation3D();

	/**
	 * Creates a new instance.
	 */
	public BoundingBox3D() {
		super();
	}

	/**
	 * Creates a new instance.
	 * 
	 * @param polygon
	 *            The borders of the plane.
	 */
	public BoundingBox3D(BoundingBox3D other) {
		super();
		setGenerator(other.getGenerator());
		try {
			setSourceDocument(other.getSourceDocument());
		} catch (NoSourceDocumentException e) {
			// nothing
		}
		setTimestamp(other.getTimestamp());
		size = new Point3D(other.size);
		origin = new Point3D(other.origin);
		rotation = new Rotation3D(other.rotation);
	}

	/**
	 * Creates a new instance.
	 * 
	 * @param origin
	 *            The origin of the plane
	 * @param rotation
	 *            The rotation of the plane's axes to the coordinate system's
	 *            axes. The z-axis is the normal of the plane
	 * @param polygon
	 *            The borders of the plane defined in the x-y axes of the plane
	 */
	public BoundingBox3D(Point3D origin, Point3D size, Rotation3D rotation) {
		super();
		this.origin = origin;
		this.size = size;
		this.rotation = rotation;
	}

	public Point3D getOrigin() {
		return origin;
	}

	public void setOrigin(Point3D origin) {
		this.origin = origin;
	}

	public Point3D getSize() {
		return size;
	}

	public void setSize(Point3D size) {
		this.size = size;
	}

	public Rotation3D getRotation() {
		return rotation;
	}

	public void setRotation(Rotation3D rotation) {
		this.rotation = rotation;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof BoundingBox3D) {
			BoundingBox3D other = (BoundingBox3D) obj;
			return size.equals(other.size) && origin.equals(other.origin)
					&& rotation.equals(other.rotation);
		}
		return false;
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "BOUNDINGBOX3D";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		// cleanup
		Elements elements = parent.getChildElements(ORIGIN_TAG_NAME);
		for (int i = 0; i < elements.size(); i++) {
			parent.removeChild(elements.get(i));
		}
		elements = parent.getChildElements(ROTATION_TAG_NAME);
		for (int i = 0; i < elements.size(); i++) {
			parent.removeChild(elements.get(i));
		}
		elements = parent.getChildElements(SIZE_TAG_NAME);
		for (int i = 0; i < elements.size(); i++) {
			parent.removeChild(elements.get(i));
		}

		// fill
		Element originElement0 = new Element(ORIGIN_TAG_NAME);
		Element originElement1 = new Element(Point3D.getBaseTag());
		origin.fillInto(originElement1);
		originElement0.appendChild(originElement1);
		parent.appendChild(originElement0);

		Element rotElement = new Element(ROTATION_TAG_NAME);
		Element rotElement1 = new Element(Rotation3D.getBaseTag());
		rotation.fillInto(rotElement1);
		rotElement.appendChild(rotElement1);
		parent.appendChild(rotElement);

		Element sizeElement = new Element(SIZE_TAG_NAME);
		Element sizeElement1 = new Element(Point3D.getBaseTag());
		size.fillInto(sizeElement1);
		sizeElement.appendChild(sizeElement1);
		parent.appendChild(sizeElement);
	}

	/**
	 * Fills an {@link BoundingBox3D} object from a given XOM {@link Element}.
	 * 
	 * @param hypothesisElement
	 *            The XOM {@link Element} to fill an object from.
	 * @param type
	 *            The {@link BoundingBox3D} object to fill with all the
	 *            information given by the {@link Element} object.
	 * @throws ParsingException
	 */
	public static void fromElement(Element dataElement, BoundingBox3D type)
			throws ParsingException {

		try {
			Type.fromElement(dataElement, type);

			Element originElement = dataElement
					.getFirstChildElement(ORIGIN_TAG_NAME);
			Element originElement1 = originElement
					.getFirstChildElement(Point3D.getBaseTag());
			Point3D origin = new Point3D();
			Point3D.fromElement(originElement1, origin);
			type.setOrigin(origin);

			Element sizeElement = dataElement
					.getFirstChildElement(SIZE_TAG_NAME);
			Element sizeElement1 = sizeElement
					.getFirstChildElement(Point3D.getBaseTag());
			Point3D size = new Point3D();
			Point3D.fromElement(sizeElement1, size);
			type.setSize(size);

			Element rotElement = dataElement
					.getFirstChildElement(ROTATION_TAG_NAME);
			Element rotElement1 = rotElement
					.getFirstChildElement(Rotation3D.getBaseTag());
			if (rotElement1 == null) {
				throw new ParsingException("Missing tag " + ROTATION_TAG_NAME);
			}
			Rotation3D rotation = new Rotation3D();
			Rotation3D.fromElement(rotElement1, rotation);
			type.setRotation(rotation);

		} catch (IllegalArgumentException e) {
			throw new ParsingException(e.getMessage(), e);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "[" + getBaseTag() + " timestamp: " + getTimestamp()
				+ " Origin: " + origin + " Rotation: " + rotation + " Size: "
				+ size + "]";

	}
}
