/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.navigation;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Nodes;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.units.*;

/**
 * Base class for all sensor results containing (robot/world) positions (X, Y,
 * theta).
 */
public class PositionData extends Type {

	private double X;
	private double Y;
	private double Yaw;
	private LengthUnit iLU = LengthUnit.METER;
	private AngleUnit iAU = AngleUnit.RADIAN;

	/**
	 * Creates a position data object initialized with x, y and theta set to 0.0
	 * and the timestamp set to the current time.
	 */
	public PositionData() {
		this(0.0, 0.0, 0.0, System.currentTimeMillis(), LengthUnit.METER,
				AngleUnit.RADIAN);
	}

	/**
	 * Creates a new position data object initialized with the given values.
	 * 
	 * @param x
	 *            The X coordinate
	 * @param y
	 *            The Y coordinate
	 * @param yaw
	 *            The yaw/rotation
	 * @param timestamp
	 *            The timestamp of this position object [ms since 1970]
	 * @param lU
	 *            length unit of input values for x and y
	 * @param aU
	 *            angle unit of input values for yaw
	 */
	public PositionData(double x, double y, double yaw, Timestamp timestamp,
			LengthUnit lU, AngleUnit aU) {
		this.X = UnitConverter.convert(x, lU, iLU);
		this.Y = UnitConverter.convert(y, lU, iLU);
		this.Yaw = UnitConverter.convert(yaw, aU, iAU);
		setTimestamp(timestamp);
	}

	public PositionData(double x, double y, double yaw, long timestamp,
			LengthUnit lU, AngleUnit aU) {
		this.X = UnitConverter.convert(x, lU, iLU);
		this.Y = UnitConverter.convert(y, lU, iLU);
		this.Yaw = UnitConverter.convert(yaw, aU, iAU);
		setTimestamp(timestamp);
	}

	/**
	 * A copy-constructor constructs a new PositionData from the passed one.
	 * 
	 * @param other
	 */
	public PositionData(PositionData other) {
		this.X = other.X;
		this.Y = other.Y;
		this.Yaw = other.Yaw;
		setTimestamp(other.getTimestamp());
	}

	/**
	 * Sets the current x Position.
	 */
	public void setX(double X, LengthUnit lU) {
		this.X = UnitConverter.convert(X, lU, iLU);
	}

	/**
	 * Sets the current y Position.
	 */
	public void setY(double Y, LengthUnit lU) {
		this.Y = UnitConverter.convert(Y, lU, iLU);
	}

	/**
	 * Sets the current yaw.
	 */
	public void setYaw(double Yaw, AngleUnit aU) {
		this.Yaw = UnitConverter.convert(Yaw, aU, iAU);
	}

	/**
	 * Returns the current x Position of the robot in world coordinates.
	 * 
	 * @return The x position as double.
	 * 
	 * @param lU
	 *            desired length unit
	 */
	public double getX(LengthUnit lU) {
		return UnitConverter.convert(X, iLU, lU);
	}

	/**
	 * Returns the current y Position of the robot in world coordinates
	 * 
	 * @return The y position as double.
	 * 
	 * @param lU
	 *            desired length unit
	 */
	public double getY(LengthUnit lU) {
		return UnitConverter.convert(Y, iLU, lU);
	}

	/**
	 * Distance to the other position in meter.
	 * 
	 * @param otherPosition
	 *            to calculate distance
	 * @return distance between goals in meter
	 * @param lU
	 *            desired length unit
	 */
	public double getDistance(PositionData otherPosition, LengthUnit lU) {
		double dx = X - otherPosition.getX(lU);
		double dy = Y - otherPosition.getY(lU);
		return Math.sqrt(Math.pow(dx, 2) + Math.pow(dy, 2));
	}

	public double getGlobalAngle(PositionData otherPosition) {
		double dx = otherPosition.getX(LengthUnit.METER) - X;
		double dy = otherPosition.getY(LengthUnit.METER) - Y;
		return Math.atan2(dy, dx);
	}

	public double getRelativeAngle(PositionData otherPosition) {
		double angle = getGlobalAngle(otherPosition) - Yaw;
		if (angle > Math.PI) {
			angle -= 2 * Math.PI;
		}
		if (angle < -Math.PI) {
			angle += 2 * Math.PI;
		}
		return angle;
	}

	/**
	 * Returns the current yaw [rad] of the robot in world coordinates.
	 * 
	 * Ranges from -PI to PI. 0.0 is returned of the robot is directed to the
	 * right. Positive values are returned if the robot is directed up and
	 * negative values if the robot is directed down.
	 * 
	 * @return The yaw as a double.
	 * 
	 * @param aU
	 *            desired angle unit
	 */
	public double getYaw(AngleUnit aU) {
		return UnitConverter.convert(Yaw, iAU, aU);
	}

	public static String getBaseTag() {
		return "ROBOTPOSITION";
	}

	public static void fromElement(Element element, PositionData type)
			throws ParsingException {

		Type.fromElement(element, type);

		Nodes positionNodes = element.query("POSITION");
		if (positionNodes.size() < 1
				&& !(positionNodes.get(0) instanceof Element)) {
			throw new ParsingException(
					"Unable to find element with name POSITION.");
		}
		Element positionElement = (Element) positionNodes.get(0);

		type.setX(ElementParser.getDoubleAttributeValue(positionElement, "x"),
				LengthUnit.METER);
		type.setY(ElementParser.getDoubleAttributeValue(positionElement, "y"),
				LengthUnit.METER);
		type.setYaw(
				ElementParser.getDoubleAttributeValue(positionElement, "theta"),
				AngleUnit.RADIAN);

	}

	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		Element positionElement = new Element("POSITION");
		parent.appendChild(positionElement);

		positionElement.addAttribute(new Attribute("x", String
				.valueOf(getX(LengthUnit.METER))));
		positionElement.addAttribute(new Attribute("y", String
				.valueOf(getY(LengthUnit.METER))));
		positionElement.addAttribute(new Attribute("theta", String
				.valueOf(getYaw(AngleUnit.RADIAN))));

		positionElement.addAttribute(new Attribute("ref", "world"));
		positionElement.addAttribute(new Attribute("kind", "absolute"));

	}

	@Override
	public boolean equals(Object obj) {

		if (!(obj instanceof PositionData)) {
			return false;
		}

		PositionData other = (PositionData) obj;

		return super.equals(obj)
				&& close(other.getX(LengthUnit.METER), getX(LengthUnit.METER))
				&& close(other.getY(LengthUnit.METER), getY(LengthUnit.METER))
				&& close(other.getYaw(AngleUnit.RADIAN),
						getYaw(AngleUnit.RADIAN));

	}

	protected boolean close(double d1, double d2) {
		return Math.abs(d1 - d2) < 0.0001;
	}

	@Override
	public String toString() {
		return "PositionData [X=" + X + ", Y=" + Y + ", Yaw=" + Yaw + ", iLU="
				+ iLU + ", iAU=" + iAU + "]";
	}

}
