/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.navigation;

import de.unibi.airobots.btl.Type;
import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Nodes;
import nu.xom.ParsingException;

/**
 * This class represents a vector consisting of configuration values. The values
 * set to each parameter of the configuration allow for manipulation of the
 * robots path planning. Receivers of this type need to be able to forward it to
 * a ROS dynamic_reconfiguration server (e.g.
 * /move_base/set_parameters) as is ros4xcf. An explanation on the
 * wide range of parameters can be found in section 3.2 here:
 * http://www.ros.org/wiki/move_base
 *
 * The purpose of sending this type is to change certain parameters of a
 * move_base node while it is already running, allowing for on the fly robot
 * recovery behavior and path-planning parameter change/adaption.
 *
 * @author pdressel
 */
public class ROSMoveBaseConfiguration extends ROSDynamicConfiguration {
    
    private static final String MOVEBASE_CONFIGURATION_TAG = "MOVEBASECONFIGURATION";
    
    public static final String BASE_GLOBAL_PLANNER_ATTRIBUTE = "base_global_planner";
    public static final String BASE_LOCAL_PLANNER_ATTRIBUTE = "base_local_planner";
    public static final String PLANNER_FREQUENCY_ATTRIBUTE = "planner_frequency";
    public static final String CONTROLLER_FREQUENCY_ATTRIBUTE = "controller_frequency";
    public static final String PLANNER_PATIENCE_ATTRIBUTE = "planner_patience";
    public static final String CONTROLLER_PATIENCE_ATTRIBUTE = "controller_patience";
    public static final String CONSERVATIVE_RESET_DIST_ATTRIBUTE = "conservative_reset_dist";
    public static final String RECOVERY_BEHAVIOR_ENABLED_ATTRIBUTE = "recovery_behavior_enabled";
    public static final String CLEARING_ROTATION_ALLOWED_ATTRIBUTE = "clearing_rotation_allowed";
    public static final String SHUTDOWN_COSTMAPS_ATTRIBUTE = "shutdown_costmaps";
    public static final String OSCILLATION_TIMEOUT_ATTRIBUTE = "oscillation_timeout";
    public static final String OSCILLATION_DISTANCE_ATTRIBUTE = "oscillation_distance";
    public static final String RESTORE_DEFAULTS_ATTRIBUTE = "restore_defaults";

    
    /**
     * Explanation: The name of the plugin for the global planner to use with 
     * move_base, see pluginlib documentation for more details on plugins. This 
     * plugin must adhere to the nav_core::BaseGlobalPlanner interface specified 
     * in the nav_core package.<br>
     * name: '<node_name>/base_global_planner'.<br>
     * Type and default: string, 'navfn/NavfnROS'
     */
    private String base_global_planner;

    /**
     * Explanation: The name of the plugin for the local planner to use with move_base see 
     * pluginlib documentation for more details on plugins. This plugin must 
     * adhere to the nav_core::BaseLocalPlanner interface specified in the 
     * nav_core package.<br>
     * name: '<node_name>/base_local_planner'.<br>
     * Type and default: string, 'base_local_planner/TrajectoryPlannerROS'
     */
    private String base_local_planner;
    
    /**
     * Explanation: The rate in Hz at which to run the global planning loop. If the frequency
     * is set to 0.0, the global planner will only run when a new goal is 
     * received or the local planner reports that its path is blocked.<br>
     * name: '<node_name>/planner_frequency'.<br>
     * Type and default: double, 0.0.
     */
    private double planner_frequency;

    /**
     * Explanation: The rate in Hz at which to run the control loop and send velocity 
     * commands to the base.<br>
     * name: '<node_name>/controller_frequency'.<br>
     * Type and default: double, 20.0
     */    
    private double controller_frequency;
    
    /**
     * Explanation: How long the planner will wait in seconds in an attempt to find a valid 
     * plan before space-clearing operations are performed.<br>
     * name: '<node_name>/planner_patience'.<br>
     * Type and default: double, 5.0
     */    
    private double planner_patience;

    /**
     * Explanation: How long the controller will wait in seconds without receiving a valid 
     * control before space-clearing operations are performed.<br>
     * name: '<node_name>/controller_patience'.<br>
     * Type and default: double, 15.0
     */     
    private double controller_patience;

    /**
     * Explanation: The distance away from the robot in meters at which obstacles will be 
     * cleared from the costmap when attempting to clear space in the map. Note,
     * this parameter is only used when the default recovery behaviors are used 
     * for move_base.<br>
     * name: '<node_name>/conservative_reset_dist'.<br>
     * Type and default: double, 3.0
     */    
    private double conservative_reset_dist;
    
    /**
     * Explanation: Whether or not to enable the move_base recovery behaviors to attempt to 
     * clear out space.<br>
     * name: '<node_name>/recovery_behavior_enabled'.<br>
     * Type and default: bool, true
     */    
    private boolean recovery_behavior_enabled;

    /**
     * Explanation: Determines whether or not the robot will attempt an in-place rotation 
     * when attempting to clear out space. Note: This parameter is only used 
     * when the default recovery behaviors are in use, meaning the user has not 
     * set the recovery_behaviors parameter to anything custom. <br>
     * name: '<node_name>/clearing_rotation_allowed'.<br>
     * Type and default: bool, true
     */  
    private boolean clearing_rotation_allowed;
    
    /**
     * Explanation: Determines whether or not to shutdown the costmaps of the node when 
     * move_base is in an inactive state.<br>
     * name: '<node_name>/shutdown_costmaps'.<br>
     * Type and default: bool, false
     */    
    private boolean shutdown_costmaps;
    
    /**
     * Explanation: How long in seconds to allow for oscillation before executing recovery 
     * behaviors. A value of 0.0 corresponds to an infinite timeout.<br>
     * name: '<node_name>/oscillation_timeoutr'.<br>
     * Type and default: double, 0.0
     */    
    private double oscillation_timeout;
    
    /**
     * Explanation: How far in meters the robot must move to be considered not to be 
     * oscillating. Moving this far resets the timer counting up to the 
     * ~oscillation_timeout.<br>
     * name: '<node_name>/oscillation_distance'.<br>
     * Type and default: double, 0.5
     */    
    private double oscillation_distance;
    
    /**
     * Explanation: Reset all values at the target dynamic_reconfiguration
     * server to its defaults. ROS topic name: '<node_name>/restore_defaults'.
     * Type and default: bool, false
     */ 
    private boolean restore_defaults;
    
    public ROSMoveBaseConfiguration()
    {
        // Fill all the properties with values from the current configuration as
        // is on the ROS server.
        // DO THIS IN BonSAI  
    }

    /**
     * Explanation: The name of the plugin for the global planner to use with
     * move_base, see pluginlib documentation for more details on plugins. This
     * plugin must adhere to the nav_core::BaseGlobalPlanner interface specified
     * in the nav_core package.<br>
     * name: '<node_name>/base_global_planner'.<br>
     * Type and default: string, 'navfn/NavfnROS'
     * @return the base_global_planner
     */
    public String getBaseGlobalPlanner() {
        return base_global_planner;
    }

    /**
     * Explanation: The name of the plugin for the global planner to use with
     * move_base, see pluginlib documentation for more details on plugins. This
     * plugin must adhere to the nav_core::BaseGlobalPlanner interface specified
     * in the nav_core package.<br>
     * name: '<node_name>/base_global_planner'.<br>
     * Type and default: string, 'navfn/NavfnROS'
     * @param base_global_planner the base_global_planner to set
     */
    public void setBaseGlobalPlanner(String base_global_planner) {
        this.base_global_planner = base_global_planner;
    }

    /**
     * Explanation: The name of the plugin for the local planner to use with move_base see
     * pluginlib documentation for more details on plugins. This plugin must
     * adhere to the nav_core::BaseLocalPlanner interface specified in the
     * nav_core package.<br>
     * name: '<node_name>/base_local_planner'.<br>
     * Type and default: string, 'base_local_planner/TrajectoryPlannerROS'
     * @return the base_local_planner
     */
    public String getBaseLocalPlanner() {
        return base_local_planner;
    }

    /**
     * Explanation: The name of the plugin for the local planner to use with move_base see
     * pluginlib documentation for more details on plugins. This plugin must
     * adhere to the nav_core::BaseLocalPlanner interface specified in the
     * nav_core package.<br>
     * name: '<node_name>/base_local_planner'.<br>
     * Type and default: string, 'base_local_planner/TrajectoryPlannerROS'
     * @param base_local_planner the base_local_planner to set
     */
    public void setBaseLocalPlanner(String base_local_planner) {
        this.base_local_planner = base_local_planner;
    }

    /**
     * Explanation: The rate in Hz at which to run the global planning loop. If the frequency
     * is set to 0.0, the global planner will only run when a new goal is
     * received or the local planner reports that its path is blocked.<br>
     * name: '<node_name>/planner_frequency'.<br>
     * Type and default: double, 0.0.
     * @return the planner_frequency
     */
    public double getPlannerFrequency() {
        return planner_frequency;
    }

    /**
     * Explanation: The rate in Hz at which to run the global planning loop. If the frequency
     * is set to 0.0, the global planner will only run when a new goal is
     * received or the local planner reports that its path is blocked.<br>
     * name: '<node_name>/planner_frequency'.<br>
     * Type and default: double, 0.0.
     * @param planner_frequency the planner_frequency to set
     */
    public void setPlannerFrequency(double planner_frequency) {
        this.planner_frequency = planner_frequency;
    }

    /**
     * Explanation: The rate in Hz at which to run the control loop and send velocity
     * commands to the base.<br>
     * name: '<node_name>/controller_frequency'.<br>
     * Type and default: double, 20.0
     * @return the controller_frequency
     */
    public double getControllerFrequency() {
        return controller_frequency;
    }

    /**
     * Explanation: The rate in Hz at which to run the control loop and send velocity
     * commands to the base.<br>
     * name: '<node_name>/controller_frequency'.<br>
     * Type and default: double, 20.0
     * @param controller_frequency the controller_frequency to set
     */
    public void setControllerFrequency(double controller_frequency) {
        this.controller_frequency = controller_frequency;
    }

    /**
     * Explanation: How long the planner will wait in seconds in an attempt to find a valid
     * plan before space-clearing operations are performed.<br>
     * name: '<node_name>/planner_patience'.<br>
     * Type and default: double, 5.0
     * @return the planner_patience
     */
    public double getPlannerPatience() {
        return planner_patience;
    }

    /**
     * Explanation: How long the planner will wait in seconds in an attempt to find a valid
     * plan before space-clearing operations are performed.<br>
     * name: '<node_name>/planner_patience'.<br>
     * Type and default: double, 5.0
     * @param planner_patience the planner_patience to set
     */
    public void setPlannerPatience(double planner_patience) {
        this.planner_patience = planner_patience;
    }

    /**
     * Explanation: How long the controller will wait in seconds without receiving a valid
     * control before space-clearing operations are performed.<br>
     * name: '<node_name>/controller_patience'.<br>
     * Type and default: double, 15.0
     * @return the controller_patience
     */
    public double getControllerPatience() {
        return controller_patience;
    }

    /**
     * Explanation: How long the controller will wait in seconds without receiving a valid
     * control before space-clearing operations are performed.<br>
     * name: '<node_name>/controller_patience'.<br>
     * Type and default: double, 15.0
     * @param controller_patience the controller_patience to set
     */
    public void setControllerPatience(double controller_patience) {
        this.controller_patience = controller_patience;
    }

    /**
     * Explanation: The distance away from the robot in meters at which obstacles will be
     * cleared from the costmap when attempting to clear space in the map. Note,
     * this parameter is only used when the default recovery behaviors are used
     * for move_base.<br>
     * name: '<node_name>/conservative_reset_dist'.<br>
     * Type and default: double, 3.0
     * @return the conservative_reset_dist
     */
    public double getConservativeResetDist() {
        return conservative_reset_dist;
    }

    /**
     * Explanation: The distance away from the robot in meters at which obstacles will be
     * cleared from the costmap when attempting to clear space in the map. Note,
     * this parameter is only used when the default recovery behaviors are used
     * for move_base.<br>
     * name: '<node_name>/conservative_reset_dist'.<br>
     * Type and default: double, 3.0
     * @param conservative_reset_dist the conservative_reset_dist to set
     */
    public void setConservativeResetDist(double conservative_reset_dist) {
        this.conservative_reset_dist = conservative_reset_dist;
    }

    /**
     * Explanation: Whether or not to enable the move_base recovery behaviors to attempt to
     * clear out space.<br>
     * name: '<node_name>/recovery_behavior_enabled'.<br>
     * Type and default: bool, true
     * @return the recovery_behavior_enabled
     */
    public boolean isRecoveryBehaviorEnabled() {
        return recovery_behavior_enabled;
    }

    /**
     * Explanation: Whether or not to enable the move_base recovery behaviors to attempt to
     * clear out space.<br>
     * name: '<node_name>/recovery_behavior_enabled'.<br>
     * Type and default: bool, true
     * @param recovery_behavior_enabled the recovery_behavior_enabled to set
     */
    public void setRecoveryBehaviorEnabled(boolean recovery_behavior_enabled) {
        this.recovery_behavior_enabled = recovery_behavior_enabled;
    }

    /**
     * Explanation: Determines whether or not the robot will attempt an in-place rotation
     * when attempting to clear out space. Note: This parameter is only used
     * when the default recovery behaviors are in use, meaning the user has not
     * set the recovery_behaviors parameter to anything custom. <br>
     * name: '<node_name>/clearing_rotation_allowed'.<br>
     * Type and default: bool, true
     * @return the clearing_rotation_allowed
     */
    public boolean isClearingRotationAllowed() {
        return clearing_rotation_allowed;
    }

    /**
     * Explanation: Determines whether or not the robot will attempt an in-place rotation
     * when attempting to clear out space. Note: This parameter is only used
     * when the default recovery behaviors are in use, meaning the user has not
     * set the recovery_behaviors parameter to anything custom. <br>
     * name: '<node_name>/clearing_rotation_allowed'.<br>
     * Type and default: bool, true
     * @param clearing_rotation_allowed the clearing_rotation_allowed to set
     */
    public void setClearingRotationAllowed(boolean clearing_rotation_allowed) {
        this.clearing_rotation_allowed = clearing_rotation_allowed;
    }

    /**
     * Explanation: Determines whether or not to shutdown the costmaps of the node when
     * move_base is in an inactive state.<br>
     * name: '<node_name>/shutdown_costmaps'.<br>
     * Type and default: bool, false
     * @return the shutdown_costmaps
     */
    public boolean isShutdownCostmaps() {
        return shutdown_costmaps;
    }

    /**
     * Explanation: Determines whether or not to shutdown the costmaps of the node when
     * move_base is in an inactive state.<br>
     * name: '<node_name>/shutdown_costmaps'.<br>
     * Type and default: bool, false
     * @param shutdown_costmaps the shutdown_costmaps to set
     */
    public void setShutdownCostmaps(boolean shutdown_costmaps) {
        this.shutdown_costmaps = shutdown_costmaps;
    }

    /**
     * Explanation: How long in seconds to allow for oscillation before executing recovery
     * behaviors. A value of 0.0 corresponds to an infinite timeout.<br>
     * name: '<node_name>/oscillation_timeoutr'.<br>
     * Type and default: double, 0.0
     * @return the oscillation_timeout
     */
    public double getOscillationTimeout() {
        return oscillation_timeout;
    }

    /**
     * Explanation: How long in seconds to allow for oscillation before executing recovery
     * behaviors. A value of 0.0 corresponds to an infinite timeout.<br>
     * name: '<node_name>/oscillation_timeoutr'.<br>
     * Type and default: double, 0.0
     * @param oscillation_timeout the oscillation_timeout to set
     */
    public void setOscillationTimeout(double oscillation_timeout) {
        this.oscillation_timeout = oscillation_timeout;
    }

    /**
     * Explanation: How far in meters the robot must move to be considered not to be
     * oscillating. Moving this far resets the timer counting up to the
     * ~oscillation_timeout.<br>
     * name: '<node_name>/oscillation_distance'.<br>
     * Type and default: double, 0.5
     * @return the oscillation_distance
     */
    public double getOscillationDistance() {
        return oscillation_distance;
    }

    /**
     * Explanation: How far in meters the robot must move to be considered not to be
     * oscillating. Moving this far resets the timer counting up to the
     * ~oscillation_timeout.<br>
     * name: '<node_name>/oscillation_distance'.<br>
     * Type and default: double, 0.5
     * @param oscillation_distance the oscillation_distance to set
     */
    public void setOscillationDistance(double oscillation_distance) {
        this.oscillation_distance = oscillation_distance;
    }

    /**
     * Explanation: Reset all values at the target dynamic_reconfiguration
     * server to its defaults. ROS topic name: '<node_name>/restore_defaults'.
     * Type and default: bool, false
     * @return the restore_defaults
     */
    public boolean isRestoreDefaults() {
        return restore_defaults;
    }

    /**
     * Explanation: Reset all values at the target dynamic_reconfiguration
     * server to its defaults. ROS topic name: '<node_name>/restore_defaults'.
     * Type and default: bool, false
     * @param restore_defaults the restore_defaults to set
     */
    public void setRestoreDefaults(boolean restore_defaults) {
        this.restore_defaults = restore_defaults;
    }
    
    /**
     * Fills an {@link ROSMoveBaseConfiguration} object from a given XOM {@link Element}
     *
     * @param objectElement The XOM {@link Element} to fill an object from.
     * @param type The {@link ROSCostmapConfiguration} object to fill with all
     * the information given by the {@link Element} object.
     */
    public static void fromElement(Element objectElement, ROSMoveBaseConfiguration type)
            throws ParsingException {

        Type.fromElement(objectElement, type);

        try {

            Nodes positionNodes = objectElement.query(MOVEBASE_CONFIGURATION_TAG);
            if (positionNodes.size() < 1
                    && !(positionNodes.get(0) instanceof Element)) {
                throw new ParsingException(
                        "Unable to find element with name " + MOVEBASE_CONFIGURATION_TAG + ".");
            }

            Element temp = (Element) positionNodes.get(0);

            type.base_global_planner = temp.getAttribute(BASE_GLOBAL_PLANNER_ATTRIBUTE).getValue();
            type.base_local_planner = temp.getAttribute(BASE_LOCAL_PLANNER_ATTRIBUTE).getValue();
            type.clearing_rotation_allowed = Boolean.parseBoolean(temp.getAttribute(CLEARING_ROTATION_ALLOWED_ATTRIBUTE).getValue());
            type.conservative_reset_dist = Double.parseDouble(temp.getAttribute(CONSERVATIVE_RESET_DIST_ATTRIBUTE).getValue());
            type.controller_frequency = Double.parseDouble(temp.getAttribute(CONTROLLER_FREQUENCY_ATTRIBUTE).getValue());
            type.controller_patience = Double.parseDouble(temp.getAttribute(CONTROLLER_PATIENCE_ATTRIBUTE).getValue());
            type.oscillation_distance = Double.parseDouble(temp.getAttribute(OSCILLATION_DISTANCE_ATTRIBUTE).getValue());
            type.oscillation_timeout = Double.parseDouble(temp.getAttribute(OSCILLATION_TIMEOUT_ATTRIBUTE).getValue());
            type.planner_frequency = Double.parseDouble(temp.getAttribute(PLANNER_FREQUENCY_ATTRIBUTE).getValue());
            type.planner_patience = Double.parseDouble(temp.getAttribute(PLANNER_PATIENCE_ATTRIBUTE).getValue());            
            type.recovery_behavior_enabled = Boolean.parseBoolean(temp.getAttribute(RECOVERY_BEHAVIOR_ENABLED_ATTRIBUTE).getValue());
            type.restore_defaults = Boolean.parseBoolean(temp.getAttribute(RESTORE_DEFAULTS_ATTRIBUTE).getValue());
            type.shutdown_costmaps = Boolean.parseBoolean(temp.getAttribute(SHUTDOWN_COSTMAPS_ATTRIBUTE).getValue());

        } catch (NullPointerException ex) {
            // this happens when an element or attribute that is required is
            // not present
            throw new IllegalArgumentException("Missing element or attribute "
                    + "in document.", ex);
        } catch (NumberFormatException e) {
            throw new IllegalArgumentException("Can not parse number", e);
        }
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public void fillInto(Element parent) {

        sanitizeElement(parent);
        super.fillInto(parent);

        Element configuration = new Element(MOVEBASE_CONFIGURATION_TAG);

        configuration.addAttribute(new Attribute(BASE_GLOBAL_PLANNER_ATTRIBUTE, String.valueOf(this.base_global_planner)));        
        configuration.addAttribute(new Attribute(BASE_LOCAL_PLANNER_ATTRIBUTE, String.valueOf(this.base_local_planner)));
        configuration.addAttribute(new Attribute(CLEARING_ROTATION_ALLOWED_ATTRIBUTE, String.valueOf(this.clearing_rotation_allowed)));
        configuration.addAttribute(new Attribute(CONSERVATIVE_RESET_DIST_ATTRIBUTE, String.valueOf(this.conservative_reset_dist)));
        configuration.addAttribute(new Attribute(CONTROLLER_FREQUENCY_ATTRIBUTE, String.valueOf(this.controller_frequency)));
        configuration.addAttribute(new Attribute(CONTROLLER_PATIENCE_ATTRIBUTE, String.valueOf(this.controller_patience)));
        configuration.addAttribute(new Attribute(OSCILLATION_DISTANCE_ATTRIBUTE, String.valueOf(this.oscillation_distance)));
        configuration.addAttribute(new Attribute(OSCILLATION_TIMEOUT_ATTRIBUTE, String.valueOf(this.oscillation_timeout)));
        configuration.addAttribute(new Attribute(PLANNER_FREQUENCY_ATTRIBUTE, String.valueOf(this.planner_frequency)));
        configuration.addAttribute(new Attribute(PLANNER_PATIENCE_ATTRIBUTE, String.valueOf(this.planner_patience)));
        configuration.addAttribute(new Attribute(RECOVERY_BEHAVIOR_ENABLED_ATTRIBUTE, String.valueOf(this.recovery_behavior_enabled)));
        configuration.addAttribute(new Attribute(RESTORE_DEFAULTS_ATTRIBUTE, String.valueOf(this.restore_defaults)));
        configuration.addAttribute(new Attribute(SHUTDOWN_COSTMAPS_ATTRIBUTE, String.valueOf(this.shutdown_costmaps)));
        
        parent.appendChild(configuration);
    }    
}
