/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.speechrec;


import java.util.ArrayList;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.Node;
import nu.xom.Nodes;

import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;

/**
 * Domain class representing a non terminal symbol within the grammartree of an
 * utterance
 * 
 * @author lschilli
 * @author sjebbara
 */
public class GrammarNonTerminal extends Type implements GrammarSymbol {

	private String name;
	private ArrayList<GrammarSymbol> subsymbols = new ArrayList<GrammarSymbol>();

	public GrammarNonTerminal() {

	}

	GrammarNonTerminal(String name) {
		this.name = name;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public GrammarSymbol getSymbol(int index) {
		return subsymbols.get(index);
	}

	public int size() {
		return subsymbols.size();
	}

	public void addSymbol(GrammarSymbol sym) {
		subsymbols.add(sym);
	}

	public ArrayList<GrammarSymbol> getSubsymbols() {
		return subsymbols;
	}

	public static String getBaseTag() {
		return "GRAMMARNONTERMINAL";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result
				+ ((subsymbols == null) ? 0 : subsymbols.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		System.out.println("compare");
		if (this == obj)
			return true;
		if (getClass() != obj.getClass()) {
			return false;
		}
		GrammarNonTerminal other = (GrammarNonTerminal) obj;
		if (name == null || name.equals("")) {
			if (other.name != null && !other.name.equals("")) {
				return false;
			}
		} else if (!name.equals(other.name)) {
			return false;
		}
		if (subsymbols == null) {
			if (other.subsymbols != null) {
				return false;
			}
		} else if (!subsymbols.equals(other.subsymbols)) {
			return false;
		}
		return true;
	}

	/** @{inheritDoc */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);
		Element ntElement = new Element("GrammarNonTerminalData");
		if(name == null) {
                    ntElement.addAttribute(new Attribute("name", ""));
                }
                else {
                    ntElement.addAttribute(new Attribute("name", name));
                }
		for (GrammarSymbol s : subsymbols) {
			if (s instanceof GrammarNonTerminal) {
				GrammarNonTerminal nt = (GrammarNonTerminal) s;
				Element subNode = new Element(GrammarNonTerminal.getBaseTag());
				nt.fillInto(subNode);
				ntElement.appendChild(subNode);
			} else if (s instanceof UtterancePart) {
				UtterancePart u = (UtterancePart) s;
				Element subNode = new Element(UtterancePart.getBaseTag());
				u.fillInto(subNode);
				ntElement.appendChild(subNode);
			}
		}
		parent.appendChild(ntElement);
	}

	/** @{inheritDoc */
	public static void fromElement(Element element, GrammarNonTerminal nt) {

		Type.fromElement(element, nt);

		Nodes nodes = element.query("GrammarNonTerminalData");
		Node ntNode = nodes.get(0);
		if (ntNode instanceof Element) {
			Element grammarSymbolElem = (Element) ntNode;
			String name = ElementParser.getAttributeValue(grammarSymbolElem,
					"name");
			nt.setName(name);
			Elements subElements = grammarSymbolElem.getChildElements();
			for (int j = 0; j < subElements.size(); j++) {
				if (subElements.get(j).getLocalName()
						.equals(UtterancePart.getBaseTag())) {
					UtterancePart u = new UtterancePart();
					UtterancePart.fromElement(subElements.get(j), u);
					nt.addSymbol(u);
				} else if (subElements.get(j).getLocalName()
						.equals(GrammarNonTerminal.getBaseTag())) {
					GrammarNonTerminal childNT = new GrammarNonTerminal();
					GrammarNonTerminal.fromElement(subElements.get(j), childNT);
					nt.addSymbol(childNT);
				}
			}
		}

	}

	@Override
	public String toString() {
		return "GrammarNonTerminal [name=" + name + ", subsymbols="
				+ subsymbols + "]";
	}

	// /**
	// * {@inheritDoc}
	// */
	// @Override
	// public String toString() {
	// StringBuilder strb = new StringBuilder();
	// strb.append(name + ":\n");
	// for (int i = 0; i < subsymbols.size(); i++) {
	// GrammarSymbol sym = subsymbols.get(i);
	// strb.append(sym + "\n");
	// }
	// return strb.toString();
	// }

}