/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.vision3d;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.data.object.ObjectLocationData;
import de.unibi.airobots.btl.data.object.ObjectLocationData.Hypothesis;
import de.unibi.airobots.btl.data.vision2d.RegionData;
import de.unibi.airobots.btl.units.AngleUnit;
import de.unibi.airobots.btl.units.UnitConverter;

public class KinectData extends Type {

	public static final String TILT_TAG_NAME = "TILT";
	public static final String ACCELERATION_TAG_NAME = "ACCELERATION";

	double tiltAngle;
	double accelerationX;
	double accelerationY;
	double accelerationZ;

	public KinectData() {
		super();
	}

	public void setTilt(double tilt, AngleUnit unit) {
		this.tiltAngle = UnitConverter.convert(tilt, unit, AngleUnit.RADIAN);
	}

	public double getTilt(AngleUnit unit) {
		return UnitConverter.convert(tiltAngle, AngleUnit.RADIAN, unit);
	}

	public void setAcceleration(double accX, double accY, double accZ) {
		this.accelerationX = accX;
		this.accelerationY = accY;
		this.accelerationZ = accZ;
	}

	public double getAccelerationX() {
		return accelerationX;
	}

	public double getAccelerationY() {
		return accelerationY;
	}

	public double getAccelerationZ() {
		return accelerationZ;
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "KINECTDATA";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		// cleanup
		Elements elements = parent.getChildElements(TILT_TAG_NAME);
		for (int i = 0; i < elements.size(); i++) {
			parent.removeChild(elements.get(i));
		}
		elements = parent.getChildElements(ACCELERATION_TAG_NAME);
		for (int i = 0; i < elements.size(); i++) {
			parent.removeChild(elements.get(i));
		}

		Element transElement = new Element(TILT_TAG_NAME);
		parent.appendChild(transElement);
		transElement.addAttribute(new Attribute("value", String
				.valueOf(tiltAngle)));

		Element rotElement = new Element(ACCELERATION_TAG_NAME);
		parent.appendChild(rotElement);
		rotElement.addAttribute(new Attribute("x", String
				.valueOf(accelerationX)));
		rotElement.addAttribute(new Attribute("y", String
				.valueOf(accelerationY)));
		rotElement.addAttribute(new Attribute("z", String
				.valueOf(accelerationZ)));
	}

	/**
	 * Fills an {@link ObjectLocationData.Hypothesis} object from a given XOM
	 * {@link Element}.
	 * 
	 * @param hypothesisElement
	 *            The XOM {@link Element} to fill an object from.
	 * @param type
	 *            The {@link ObjectLocationData.Hypothesis} object to fill with
	 *            all the information given by the {@link Element} object.
	 * @throws ParsingException
	 */
	public static void fromElement(Element dataElement, KinectData type)
			throws ParsingException {

		RegionData.fromElement(dataElement, type);

		Element transElement = dataElement.getFirstChildElement(TILT_TAG_NAME);
		double tilt = ElementParser.getDoubleAttributeValue(transElement,
				"value");
		type.setTilt(tilt, AngleUnit.RADIAN);

		Element rotElement = dataElement
				.getFirstChildElement(ACCELERATION_TAG_NAME);
		double accX = ElementParser.getDoubleAttributeValue(rotElement, "x");
		double accY = ElementParser.getDoubleAttributeValue(rotElement, "y");
		double accZ = ElementParser.getDoubleAttributeValue(rotElement, "z");
		type.setAcceleration(accX, accY, accZ);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "[" + getBaseTag() + " timestamp: " + getTimestamp() + " tilt: "
				+ tiltAngle + " acceleration: " + accelerationX + ", "
				+ accelerationY + ", " + accelerationZ + "]";

	}
}
