/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke,
 * 				Dennis Leroy Wigand
 */

/*
 * BodyTest.cpp
 *
 *  Created on: Jul 16, 2012
 *      Author: Dennis Leroy Wigand
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>
#include <math.h>

#include "btl/data/person/Body.h"
#include "btl/units/LengthUnit.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(BodyTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {
		Body original;

		Point3D p0(4.2, 1.4, 2.4, MILLIMETER);
		Point3D p1(2.7, 4.3, 7.8, MILLIMETER);

		original.setCoM(p0);
		original.setTorso(p1);
		original.setSkeleton(true);

		Location originalLoc(Body::getTemplate(), Body::getBaseTag());

		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		Body parsed = extract<Body>(originalLoc);

		BOOST_REQUIRE_EQUAL(original.getGenerator(), parsed.getGenerator());
		BOOST_REQUIRE_EQUAL(original.getTimestamp(), parsed.getTimestamp());
		BOOST_REQUIRE_CLOSE(original.getCoM().getX(MILLIMETER), parsed.getCoM().getX(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getCoM().getY(MILLIMETER), parsed.getCoM().getY(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getCoM().getZ(MILLIMETER), parsed.getCoM().getZ(MILLIMETER), TOLERANCE);

		BOOST_REQUIRE_CLOSE(original.getTorso().getX(MILLIMETER), parsed.getTorso().getX(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getTorso().getY(MILLIMETER), parsed.getTorso().getY(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getTorso().getZ(MILLIMETER), parsed.getTorso().getZ(MILLIMETER), TOLERANCE);

		BOOST_REQUIRE(original.isSkeleton() == parsed.isSkeleton());
		BOOST_REQUIRE(original.getKinectId() == parsed.getKinectId());

		BOOST_REQUIRE(original == parsed);

	} catch (std::exception &e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("Body.xml"), Body::getBaseTag());
	Body parsed = extract<Body>(fileContents);

	BOOST_REQUIRE_EQUAL("BodyTester", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE(12, parsed.getCoM().getX(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(12, parsed.getCoM().getY(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(81, parsed.getCoM().getZ(MILLIMETER), TOLERANCE);

	BOOST_REQUIRE_CLOSE(12, parsed.getTorso().getX(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(12, parsed.getTorso().getY(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(81, parsed.getTorso().getZ(MILLIMETER), TOLERANCE);

	BOOST_REQUIRE((-1) == parsed.getKinectId());
	BOOST_REQUIRE(!parsed.isSkeleton());
}

BOOST_AUTO_TEST_SUITE_END()
